/** 
 *   Site testing for SKA candidate.
 *
 *   (c) Copyright 2004 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/


#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <sys/file.h>
#include <sys/stat.h>

#include "my_stdio.h"
#include "observ_table.h"
#include "analyzer.h"


struct observ *make_observ(void)
{
	struct observ *obs;
	
	obs = (struct observ *) malloc(sizeof(struct observ));
	if( obs == NULL ){
		PVERB(PERR,"make_observ: error alocating struct observ.\n");
		return NULL;
	}
	obs->next = (struct observ *) NULL;
	obs->mode = -1;

	return obs;
}


static int get_observ(char *s, struct observ **o)
{
	int i, res;
	char mode;
	struct observ *op = *o;
	struct observ *of = *o;
	int remainder;
	double delta;
	double start_freq, stop_freq , final_freq;
	int integer;
	

	res = sscanf(s,"%c %6lf %lf %d %d %d %c",
		     &mode, &op->start_freq, &op->stop_freq, 
		     &op->RBW, &op->sweep, &op->repetitions, &op->switchs);
	if(res != 7){
		PVERB(PERR,"get_observ: Error in observation file (get %d)\n",
		      res);
		return -1;
	}      

	switch(mode) {
	case '0':
	case 'c':
	case 'C':
		op->mode = 0;
		break;
	case '1':
		op->mode = 1;
		break;
	case '2':
		op->mode = 2;
		break;
	default:
		PVERB(PERR,"get_observ: Unknow mode %c\n", mode);
		return -1;
	}

	integer = (int) ((double) op->sweep / SWEEP_MAX);

	if( integer ) {
		remainder = (int) fmod((double) op->sweep, SWEEP_MAX);
		delta = (op->stop_freq - op->start_freq ) /
			((double) integer + (double) (remainder ? 1 : 0));
		final_freq = op->stop_freq;
		op->stop_freq = op->start_freq + delta;
		op->sweep = SWEEP_MAX;
		start_freq = op->stop_freq;
		for(i=2; i <= integer  ; i++){
			op->next = make_observ(); 
			op = op->next;
			stop_freq = of->start_freq + ((double) i * delta);
			op->mode = of->mode;
			op->start_freq = start_freq;
			op->stop_freq = stop_freq;
			op->RBW = of->RBW;
			op->sweep = SWEEP_MAX;
			op->repetitions = of->repetitions;
			op->switchs = of->switchs;
			start_freq = stop_freq;
		}
		if( remainder ){
			op->next = make_observ();
			op = op->next;
			op->mode = of->mode;
			op->start_freq = start_freq;
			op->stop_freq = final_freq;
			op->RBW = of->RBW;
			op->sweep = remainder;
			op->repetitions = of->repetitions;
			op->switchs = of->switchs;
		}
	}
	*o = op;

	return 0;
}


struct observ *get_observ_table(char *file_name)
{
	FILE *fp_observ;
	char line[160];
	//int stop = 0;
	int start = 0;
	int i;
	struct observ *obs, *obsp;

	obsp = obs = make_observ();
	fp_observ = fopen(file_name, "r");
	if( fp_observ == NULL ){
		PVERB(PERR,
		      "get_observ_table: Error opening file: %s\n",file_name);
		free(obsp);
		return NULL;
	}
	while( fgets(line, sizeof(line), fp_observ) /*&& !stop*/){
		if( memcmp(line,"observ",6) == 0 && start){
			for( i=0; (i < 160) && (line[i] != '=');i++);
			i++;
			if( get_observ(stripwhite(line+i), &obsp) == 0 ){
				if(obsp == NULL) {
					PVERB(PERR,
				          "get_observ_table: obsp is NULL\n");
					exit(1);
				}
				obsp->next = (struct observ *) 
					malloc(sizeof(struct observ));
				obsp = obsp->next;
				obsp->next = NULL;
			}
		}else{
			if( memcmp(line,"start",5) == 0 ){
				start = 1;
			}

			if( memcmp(line,"stop",4) == 0 ){
				//stop = 1;
				start = 0;
			}
		}
	}
	fclose(fp_observ);

	return obs;
}


int free_observ_table(struct observ *obs)
{
	struct observ *op = obs;
	struct observ *opp;

	while(op != NULL){
		opp = op;
		op = op->next;
		free(opp);
	}

	return 0;
}
